@@ -47,6 +47,8 @@
 #include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
 
+static const char *none_prefix[] = { NULL };
+
 struct speedup_data {
 	GAtChat *modem;
 	GAtChat *aux;
@@ -77,14 +79,20 @@ static void speedup_remove(struct ofono_modem *modem)
 
 	ofono_modem_set_data(modem, NULL);
 
+	/* Cleanup potential SIM state polling */
+	at_util_sim_state_query_free(data->sim_state_query);
+
+	/* Cleanup after hot-unplug */
+	g_at_chat_unref(data->aux);
+
 	g_free(data);
 }
 
 static void speedup_debug(const char *str, void *user_data)
 {
-        const char *prefix = user_data;
+	const char *prefix = user_data;
 
-        ofono_info("%s%s", prefix, str);
+	ofono_info("%s%s", prefix, str);
 }
 
 static GAtChat *open_device(struct ofono_modem *modem,
@@ -135,6 +143,16 @@ static void sim_state_cb(gboolean present, gpointer user_data)
 	/* AT&C0 needs to be send separate and on both channel */
 	g_at_chat_send(data->modem, "AT&C0", NULL, NULL, NULL, NULL);
 	g_at_chat_send(data->aux, "AT&C0", NULL, NULL, NULL, NULL);
+
+	/*
+	 * Ensure that the modem is using GSM character set and not IRA,
+	 * otherwise weirdness with umlauts and other non-ASCII characters
+	 * can result
+	 */
+	g_at_chat_send(data->modem, "AT+CSCS=\"GSM\"", none_prefix,
+							NULL, NULL, NULL);
+	g_at_chat_send(data->aux, "AT+CSCS=\"GSM\"", none_prefix,
+							NULL, NULL, NULL);
 }
 
 static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
@@ -228,7 +246,7 @@ static void speedup_pre_sim(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	ofono_devinfo_create(modem, 0, "atmodem", data->aux);
-	sim = ofono_sim_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+	sim = ofono_sim_create(modem, OFONO_VENDOR_SPEEDUP,
 						"atmodem", data->aux);
 
 	if (sim && data->have_sim == TRUE)
@@ -243,25 +261,31 @@ static void speedup_post_sim(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	ofono_netreg_create(modem, 0, "atmodem", data->aux);
-
-	ofono_cbs_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
-					"atmodem", data->aux);
-	ofono_ussd_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
-					"atmodem", data->aux);
-
 	ofono_phonebook_create(modem, 0, "atmodem", data->aux);
 
 	ofono_sms_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
-					"atmodem", data->aux);
+						"atmodem", data->aux);
 
-	gprs = ofono_gprs_create(modem, 0, "atmodem", data->aux);
+	gprs = ofono_gprs_create(modem, OFONO_VENDOR_SPEEDUP,
+						"atmodem", data->aux);
 	gc = ofono_gprs_context_create(modem, 0, "atmodem", data->modem);
 
 	if (gprs && gc)
 		ofono_gprs_add_context(gprs, gc);
 }
 
+static void speedup_post_online(struct ofono_modem *modem)
+{
+	struct speedup_data *data = ofono_modem_get_data(modem);
+
+	ofono_netreg_create(modem, OFONO_VENDOR_SPEEDUP, "atmodem", data->aux);
+
+	ofono_cbs_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+						"atmodem", data->aux);
+	ofono_ussd_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+						"atmodem", data->aux);
+}
+
 static struct ofono_modem_driver speedup_driver = {
 	.name		= "speedup",
 	.probe		= speedup_probe,
@@ -270,6 +294,7 @@ static struct ofono_modem_driver speedup_driver = {
 	.disable	= speedup_disable,
 	.pre_sim	= speedup_pre_sim,
 	.post_sim	= speedup_post_sim,
+	.post_online	= speedup_post_online,
 };
 
 static int speedup_init(void)
